package com.hjsoft.base.staff;

package ..deTemp.1744628797052;

import com.hjsoft.base.staff.DeleteSubEntity;
import com.hjsoft.base.staff.DeleteSubPartEntity;
import com.hjsoft.base.staff.DeleteSubPartService;
import com.hjsoft.base.staff.DeleteSubService;
import com.hjsoft.frame.security.ShiroUser;
import com.hjsoft.frame.user.UserEntity;
import com.mongodb.AggregationOptions;
import com.mongodb.BasicDBObject;
import com.mongodb.Cursor;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.WriteResult;
import com.rap.common.entity.AutoIdEntity;
import com.rap.common.exception.ServiceException;
import com.rap.common.exception.SystemException;
import com.rap.common.jpa.document.DocRepository;
import com.rap.common.jpa.page.PageHolder;
import com.rap.common.jpa.page.VariablePageRequest;
import com.rap.common.utils.StringUtil;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import org.apache.commons.lang3.StringUtils;
import org.bson.types.ObjectId;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.query.BasicQuery;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.CriteriaDefinition;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

@Service("BasicMongoDBService")
public class BasicMongoDBService {
  Logger logger = LoggerFactory.getLogger(getClass());
  
  @Autowired
  private MongoTemplate mongoTemplate;
  
  @Autowired
  private DeleteSubService deleteSubService;
  
  @Autowired
  private DeleteSubPartService deleteSubPartService;
  
  @Autowired(required = false)
  private DocRepository docRepository;
  
  public BasicDBObject findById(String id, String subCode) {
    return (BasicDBObject)this.mongoTemplate.findById(id, BasicDBObject.class, subCode);
  }
  
  public BasicDBObject findByDdbs(String tenantId, String ddbs, String subCode) {
    Query query = new Query((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("ddbs").is(ddbs));
    return (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findByIds(String tenantId, Set<ObjectId> ids, String subCode) {
    Query query = new Query((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("_id").in(ids));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findPartBypartIds(String tenantId, Set<String> ids, String subCode) {
    Query query = new Query((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("partTimeJobId").in(ids));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findBySubAndDdbs(String tenantId, Set<String> ddbss, String subCode) {
    Query query = new Query((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("ddbs").in(ddbss));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findByIds(String tenantId, Set<ObjectId> ids, Set<String> fields, String subCode) {
    BasicDBObject basicDBObject = new BasicDBObject();
    if (fields != null && !fields.isEmpty())
      for (String f : fields)
        basicDBObject.put(f, Boolean.valueOf(true));  
    BasicQuery query = new BasicQuery((DBObject)new BasicDBObject(), (DBObject)basicDBObject);
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("_id").in(ids));
    return this.mongoTemplate.find((Query)query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findAllStaffMaxOrderBySubCode(String subCode) {
    List<BasicDBObject> list = new ArrayList<>();
    AggregationOptions build = AggregationOptions.builder().outputMode(AggregationOptions.OutputMode.CURSOR).build();
    List<DBObject> pipeline = new ArrayList<>();
    BasicDBObject basicDBObject1 = new BasicDBObject("_id", "$staffId");
    BasicDBObject basicDBObject2 = new BasicDBObject("$max", "$showOrder");
    basicDBObject1.put("maxOrder", basicDBObject2);
    BasicDBObject basicDBObject3 = new BasicDBObject();
    basicDBObject3.put("$group", basicDBObject1);
    pipeline.add(basicDBObject3);
    Cursor cursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    while (cursor.hasNext())
      list.add((BasicDBObject)cursor.next()); 
    return list;
  }
  
  public BasicDBObject save(BasicDBObject entity, String subCode) {
    UserEntity loginUser = ShiroUser.getCurrentUserEntity();
    entity = setLogs(entity, loginUser);
    if (entity.get("_id") == null && subCode.startsWith("A") && !entity.containsField("staffBase"))
      entity.append("staffBase", "0"); 
    this.mongoTemplate.save(entity, subCode);
    return entity;
  }
  
  public BasicDBObject saveOrUpdate(BasicDBObject entity, String subCode) {
    UserEntity loginUser = ShiroUser.getCurrentUserEntity();
    entity = setLogs(entity, loginUser);
    if (entity.get("_id") != null) {
      Query query = new Query();
      query.addCriteria((CriteriaDefinition)Criteria.where("_id").is(entity.getObjectId("_id")));
      Update update = new Update();
      for (String key : entity.keySet()) {
        if (StringUtil.isEquals("_id", key))
          continue; 
        update.set(key, entity.get(key));
      } 
      this.mongoTemplate.updateFirst(query, update, subCode);
    } else {
      if (subCode.startsWith("A") && !entity.containsField("staffBase"))
        entity.append("staffBase", "0"); 
      this.mongoTemplate.save(entity, subCode);
    } 
    return entity;
  }
  
  public BasicDBObject saveOrUpdateNoShiro(BasicDBObject entity, String subCode, UserEntity loginUser) {
    entity = setLogs(entity, loginUser);
    if (entity == null)
      return null; 
    if (entity.get("_id") != null) {
      Query query = new Query();
      query.addCriteria((CriteriaDefinition)Criteria.where("_id").is(entity.getObjectId("_id")));
      Update update = new Update();
      for (String key : entity.keySet()) {
        if (StringUtil.isEquals("_id", key))
          continue; 
        update.set(key, entity.get(key));
      } 
      this.mongoTemplate.updateFirst(query, update, subCode);
    } else {
      if (subCode.startsWith("A") && !entity.containsField("staffBase"))
        entity.append("staffBase", "0"); 
      this.mongoTemplate.save(entity, subCode);
    } 
    return entity;
  }
  
  public BasicDBObject saveOrUpdate(BasicDBObject entity, String subCode, String base) {
    UserEntity loginUser = ShiroUser.getCurrentUserEntity();
    entity = setLogs(entity, loginUser);
    if (entity.get("_id") != null) {
      Query query = new Query();
      query.addCriteria((CriteriaDefinition)Criteria.where("_id").is(entity.getObjectId("_id")));
      Update update = new Update();
      for (String key : entity.keySet()) {
        if (StringUtil.isEquals("_id", key))
          continue; 
        update.set(key, entity.get(key));
      } 
      this.mongoTemplate.updateFirst(query, update, subCode);
    } else {
      if (StringUtils.isNotBlank(base)) {
        entity.append("staffBase", base);
      } else if (subCode.startsWith("A") && !entity.containsField("staffBase")) {
        entity.append("staffBase", "0");
      } 
      this.mongoTemplate.save(entity, subCode);
    } 
    return entity;
  }
  
  public int update(Query query, BasicDBObject entity, String subCode) {
    Set<Map.Entry<String, Object>> es = entity.entrySet();
    Update update = new Update();
    for (Map.Entry<String, Object> e : es) {
      if (e.getValue() == null || "_id".equals(e.getKey()))
        continue; 
      update.set(e.getKey(), e.getValue());
    } 
    WriteResult wr = this.mongoTemplate.updateMulti(query, update, subCode);
    if (wr.getN() < 1 && this.logger.isWarnEnabled())
      this.logger.warn("No object been updated {} failed : {}", entity, wr.toString()); 
    return wr.getN();
  }
  
  public int updateById(BasicDBObject entity, String subCode) {
    ObjectId id = entity.getObjectId("_id");
    if (id == null)
      throw new SystemException(null, "_id is not allow null, when update by id"); 
    Set<Map.Entry<String, Object>> es = entity.entrySet();
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("_id").is(id));
    Update update = new Update();
    for (Map.Entry<String, Object> e : es) {
      if (e.getValue() == null || "_id".equals(e.getKey()))
        continue; 
      update.set(e.getKey(), e.getValue());
    } 
    WriteResult wr = this.mongoTemplate.updateFirst(query, update, subCode);
    if (wr.getN() < 1 && this.logger.isWarnEnabled())
      this.logger.warn("Update object {} failed : {}", entity, wr.toString()); 
    return wr.getN();
  }
  
  public int updatesById(List<BasicDBObject> entities, String subCode) {
    int count = 0;
    for (BasicDBObject e : entities)
      count += updateById(e, subCode); 
    return count;
  }
  
  public List<BasicDBObject> insert(List<BasicDBObject> entities, String subCode) {
    UserEntity loginUser = ShiroUser.getCurrentUserEntity();
    List<BasicDBObject> list = new ArrayList<>();
    if (entities != null) {
      Iterator<BasicDBObject> i = entities.iterator();
      while (i.hasNext()) {
        BasicDBObject t = setLogs(i.next(), loginUser);
        list.add(t);
      } 
    } 
    this.mongoTemplate.insert(list, subCode);
    return list;
  }
  
  public List<BasicDBObject> save(List<BasicDBObject> entities, String subCode) {
    List<BasicDBObject> list = new ArrayList<>();
    if (entities != null) {
      Iterator<BasicDBObject> i = entities.iterator();
      while (i.hasNext()) {
        BasicDBObject t = save(i.next(), subCode);
        list.add(t);
      } 
    } 
    return list;
  }
  
  public List<BasicDBObject> saveOrUpdate(List<BasicDBObject> entities, String subCode) {
    List<BasicDBObject> list = new ArrayList<>();
    if (entities != null) {
      Iterator<BasicDBObject> i = entities.iterator();
      while (i.hasNext()) {
        BasicDBObject t = saveOrUpdate(i.next(), subCode);
        list.add(t);
      } 
    } 
    return list;
  }
  
  public List<BasicDBObject> saveOrUpdateNoShiro(List<BasicDBObject> entities, String subCode, UserEntity loginUser) {
    List<BasicDBObject> list = new ArrayList<>();
    if (entities != null) {
      Iterator<BasicDBObject> i = entities.iterator();
      while (i.hasNext()) {
        BasicDBObject t = saveOrUpdateNoShiro(i.next(), subCode, loginUser);
        list.add(t);
      } 
    } 
    return list;
  }
  
  public BasicDBObject setLogs(BasicDBObject entity, UserEntity user) {
    if (entity == null)
      return entity; 
    if (entity.get("_id") == null) {
      entity.append("createTs", new Date());
      entity.append("creator", (user == null) ? "" : user.getUuid());
    } 
    entity.append("modifyTs", new Date());
    if (user == null)
      return entity; 
    if (StringUtil.isEmpty(entity.get("tenantId")))
      entity.append("tenantId", user.getTenantId()); 
    if (StringUtil.isEmpty(entity.get("modifier")))
      entity.append("modifier", user.getUuid()); 
    return entity;
  }
  
  public void deleteById(String id, String subCode) {
    DeleteSubEntity deleteSubEntity = new DeleteSubEntity();
    deleteSubEntity.setDeleteId(id);
    deleteSubEntity.setSub(subCode);
    try {
      this.deleteSubService.save((AutoIdEntity)deleteSubEntity);
    } catch (ServiceException e) {
      e.printStackTrace();
    } 
    BasicDBObject byUuid = (BasicDBObject)this.mongoTemplate.findById(id, BasicDBObject.class, subCode);
    DeleteSubPartEntity deleteSubEntity1 = new DeleteSubPartEntity();
    if (byUuid.get("hcmId") != null)
      deleteSubEntity1.setDeleteId(byUuid.get("hcmId").toString()); 
    deleteSubEntity1.setSub(subCode);
    if (byUuid.get("orgId") == null) {
      deleteSubEntity1.setStaffId(byUuid.get("staffId").toString());
    } else {
      deleteSubEntity1.setOrgId(byUuid.get("orgId").toString());
    } 
    try {
      this.deleteSubPartService.save((AutoIdEntity)deleteSubEntity1);
    } catch (ServiceException e) {
      e.printStackTrace();
    } 
    if (StringUtil.isEmpty(id))
      return; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("_id").is(new ObjectId(id)));
    this.mongoTemplate.remove(query, subCode);
  }
  
  public void deleteByIds(List<ObjectId> ids, String subCode) {
    if (ids == null || ids.size() == 0)
      return; 
    try {
      if (ShiroUser.getEntity() != null) {
        List<DeleteSubEntity> deleteSubEntitys = new ArrayList<>();
        List<DeleteSubPartEntity> deleteSubPartEntities = new ArrayList<>();
        Set<ObjectId> idSet = new HashSet<>(ids);
        List<BasicDBObject> basicDBObjects = findByIds(ShiroUser.getEntity().getTenantId(), idSet, subCode);
        for (BasicDBObject id : basicDBObjects) {
          DeleteSubEntity deleteSubEntity = new DeleteSubEntity();
          deleteSubEntity.setDeleteId(id.get("_id").toString());
          deleteSubEntity.setSub(subCode);
          deleteSubEntitys.add(deleteSubEntity);
          DeleteSubPartEntity deleteSubEntity1 = new DeleteSubPartEntity();
          if (id.get("hcmId") != null)
            deleteSubEntity1.setDeleteId(id.get("hcmId").toString()); 
          deleteSubEntity1.setSub(subCode);
          if (id.get("orgId") == null) {
            deleteSubEntity1.setStaffId(id.get("staffId").toString());
          } else {
            deleteSubEntity1.setOrgId(id.get("orgId").toString());
          } 
          deleteSubPartEntities.add(deleteSubEntity1);
        } 
        try {
          this.deleteSubService.save(deleteSubEntitys);
          this.deleteSubPartService.save(deleteSubPartEntities);
        } catch (ServiceException e) {
          e.printStackTrace();
        } 
      } 
    } catch (Exception e) {
      e.printStackTrace();
    } 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("_id").in(ids));
    this.mongoTemplate.remove(query, subCode);
  }
  
  public void deleteBySubCodeAndStaffIdWithNoDdbs(String staffId, String subCode) {
    if (StringUtil.isEmpty(staffId))
      return; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("staffId").is(staffId));
    query.addCriteria((CriteriaDefinition)(new Criteria()).orOperator(new Criteria[] { Criteria.where("ddbs").exists(false), 
            Criteria.where("ddbs").is(null), 
            Criteria.where("ddbs").is("") }));
    List<String> staffIdList = new ArrayList<>();
    staffIdList.add(staffId);
    List<DeleteSubEntity> deleteSubEntitys = new ArrayList<>();
    List<DeleteSubPartEntity> deleteSubPartEntities = new ArrayList<>();
    List<BasicDBObject> basicDBObjects = findBySubCodeAndStaffIdListWithNoDdbs(subCode, staffIdList);
    for (BasicDBObject id : basicDBObjects) {
      DeleteSubEntity deleteSubEntity = new DeleteSubEntity();
      deleteSubEntity.setDeleteId(id.get("_id").toString());
      deleteSubEntity.setSub(subCode);
      deleteSubEntitys.add(deleteSubEntity);
      DeleteSubPartEntity deleteSubEntity1 = new DeleteSubPartEntity();
      if (id.get("hcmId") != null)
        deleteSubEntity1.setDeleteId(id.get("hcmId").toString()); 
      deleteSubEntity1.setSub(subCode);
      if (id.get("orgId") == null) {
        deleteSubEntity1.setStaffId(id.get("staffId").toString());
      } else {
        deleteSubEntity1.setOrgId(id.get("orgId").toString());
      } 
      deleteSubPartEntities.add(deleteSubEntity1);
    } 
    try {
      this.deleteSubService.save(deleteSubEntitys);
      this.deleteSubPartService.save(deleteSubPartEntities);
    } catch (ServiceException e) {
      e.printStackTrace();
    } 
    this.mongoTemplate.remove(query, subCode);
  }
  
  public void deleteBySubCodeAndStaffId(String staffId, String subCode) {
    if (StringUtil.isEmpty(staffId))
      return; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("staffId").is(staffId));
    List<String> staffIdList = new ArrayList<>();
    staffIdList.add(staffId);
    List<DeleteSubEntity> deleteSubEntitys = new ArrayList<>();
    List<DeleteSubPartEntity> deleteSubPartEntities = new ArrayList<>();
    List<BasicDBObject> basicDBObjects = findBySubCodeAndStaffIdList(subCode, staffIdList);
    for (BasicDBObject id : basicDBObjects) {
      DeleteSubEntity deleteSubEntity = new DeleteSubEntity();
      deleteSubEntity.setDeleteId(id.get("_id").toString());
      deleteSubEntity.setSub(subCode);
      deleteSubEntitys.add(deleteSubEntity);
      DeleteSubPartEntity deleteSubEntity1 = new DeleteSubPartEntity();
      if (id.get("hcmId") != null)
        deleteSubEntity1.setDeleteId(id.get("hcmId").toString()); 
      deleteSubEntity1.setSub(subCode);
      if (id.get("orgId") == null) {
        deleteSubEntity1.setStaffId(id.get("staffId").toString());
      } else {
        deleteSubEntity1.setOrgId(id.get("orgId").toString());
      } 
      deleteSubPartEntities.add(deleteSubEntity1);
    } 
    try {
      this.deleteSubService.save(deleteSubEntitys);
      this.deleteSubPartService.save(deleteSubPartEntities);
    } catch (ServiceException e) {
      e.printStackTrace();
    } 
    this.mongoTemplate.remove(query, subCode);
  }
  
  public void deleteBySubCodeAndFieldCodeAndFieldValue(String subCode, String fieldCode, String fieldValue) {
    if (StringUtil.isEmpty(subCode) || StringUtil.isEmpty(fieldCode) || StringUtil.isEmpty(fieldValue))
      return; 
    Query query = new Query();
    if ("_id".equals(fieldCode)) {
      query.addCriteria((CriteriaDefinition)Criteria.where(fieldCode).is(new ObjectId(fieldValue)));
    } else {
      query.addCriteria((CriteriaDefinition)Criteria.where(fieldCode).is(fieldValue));
    } 
    this.mongoTemplate.remove(query, subCode);
  }
  
  public void deleteBySubCodeAndTenantId(String tenantId, String subCode) {
    if (StringUtil.isEmpty(tenantId))
      return; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId));
    this.mongoTemplate.remove(query, subCode);
  }
  
  public void deleteBySubCodeAndStaffIdList(List<String> staffIdList, String subCode) {
    List<DeleteSubEntity> deleteSubEntitys = new ArrayList<>();
    List<DeleteSubPartEntity> deleteSubPartEntities = new ArrayList<>();
    if (staffIdList == null || staffIdList.size() == 0)
      return; 
    List<BasicDBObject> basicDBObjects = findBySubCodeAndStaffIdList(subCode, staffIdList);
    for (BasicDBObject id : basicDBObjects) {
      DeleteSubEntity deleteSubEntity = new DeleteSubEntity();
      deleteSubEntity.setDeleteId(id.get("_id").toString());
      deleteSubEntity.setSub(subCode);
      deleteSubEntitys.add(deleteSubEntity);
      DeleteSubPartEntity deleteSubEntity1 = new DeleteSubPartEntity();
      if (id.get("hcmId") != null)
        deleteSubEntity1.setDeleteId(id.get("hcmId").toString()); 
      deleteSubEntity1.setSub(subCode);
      if (id.get("orgId") == null) {
        deleteSubEntity1.setStaffId(id.get("staffId").toString());
      } else {
        deleteSubEntity1.setOrgId(id.get("orgId").toString());
      } 
      deleteSubPartEntities.add(deleteSubEntity1);
    } 
    try {
      this.deleteSubService.save(deleteSubEntitys);
      this.deleteSubPartService.save(deleteSubPartEntities);
    } catch (ServiceException e) {
      e.printStackTrace();
    } 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("staffId").in(staffIdList));
    this.mongoTemplate.remove(query, subCode);
  }
  
  public void deleteBySubCodeAndStaffIdListWithDensity(List<String> staffIdList, String subCode, HashMap<String, Object> densityMap) {
    List<DeleteSubEntity> deleteSubEntitys = new ArrayList<>();
    List<BasicDBObject> basicDBObjects = findBySubCodeAndStaffIdList(subCode, staffIdList);
    List<DeleteSubPartEntity> deleteSubPartEntities = new ArrayList<>();
    for (BasicDBObject id : basicDBObjects) {
      DeleteSubEntity deleteSubEntity = new DeleteSubEntity();
      deleteSubEntity.setDeleteId(id.get("_id").toString());
      deleteSubEntity.setSub(subCode);
      deleteSubEntitys.add(deleteSubEntity);
      DeleteSubPartEntity deleteSubEntity1 = new DeleteSubPartEntity();
      if (id.get("hcmId") != null)
        deleteSubEntity1.setDeleteId(id.get("hcmId").toString()); 
      deleteSubEntity1.setSub(subCode);
      if (id.get("orgId") == null) {
        deleteSubEntity1.setStaffId(id.get("staffId").toString());
        continue;
      } 
      deleteSubEntity1.setOrgId(id.get("orgId").toString());
    } 
    try {
      this.deleteSubService.save(deleteSubEntitys);
      this.deleteSubPartService.save(deleteSubPartEntities);
    } catch (ServiceException e) {
      e.printStackTrace();
    } 
    if (staffIdList == null || staffIdList.size() == 0)
      return; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("staffId").in(staffIdList));
    if (densityMap != null && densityMap.get("isStartDensity") != null && ((Boolean)densityMap.get("isStartDensity")).booleanValue())
      query.addCriteria(
          (CriteriaDefinition)Criteria.where(densityMap.get("codeField").toString())
          .nin((List)densityMap.get("levelList"))); 
    this.mongoTemplate.remove(query, subCode);
  }
  
  public void deleteBySubCodeAndOrgIdList(List<String> orgIdList, String subCode) {
    List<DeleteSubEntity> deleteSubEntitys = new ArrayList<>();
    List<BasicDBObject> basicDBObjects = findBySubCodeAndOrgIdList(subCode, orgIdList);
    List<DeleteSubPartEntity> deleteSubPartEntities = new ArrayList<>();
    for (BasicDBObject id : basicDBObjects) {
      DeleteSubPartEntity deleteSubEntity1 = new DeleteSubPartEntity();
      if (id.get("hcmId") != null)
        deleteSubEntity1.setDeleteId(id.get("hcmId").toString()); 
      deleteSubEntity1.setSub(subCode);
      if (id.get("orgId") == null) {
        deleteSubEntity1.setStaffId(id.get("staffId").toString());
      } else {
        deleteSubEntity1.setOrgId(id.get("orgId").toString());
      } 
      DeleteSubEntity deleteSubEntity = new DeleteSubEntity();
      deleteSubEntity.setDeleteId(id.get("_id").toString());
      deleteSubEntity.setSub(subCode);
      deleteSubEntitys.add(deleteSubEntity);
    } 
    try {
      this.deleteSubPartService.save(deleteSubPartEntities);
      this.deleteSubService.save(deleteSubEntitys);
    } catch (ServiceException e) {
      e.printStackTrace();
    } 
    if (orgIdList == null || orgIdList.size() == 0)
      return; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("orgId").in(orgIdList));
    this.mongoTemplate.remove(query, subCode);
  }
  
  public void deleteBySubCodeAndTenantIdAndStaffIdList(String subCode, String tenantId, List<String> staffIdList) {
    if (StringUtil.isEmpty(tenantId))
      return; 
    if (staffIdList == null || staffIdList.size() == 0)
      return; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").in(staffIdList));
    this.mongoTemplate.remove(query, subCode);
  }
  
  public void deleteByThirdIdsAndSubCode(List<String> thirdIds, String subCode) {
    if (thirdIds == null || thirdIds.size() == 0)
      return; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("thirdId").in(thirdIds));
    this.mongoTemplate.remove(query, subCode);
  }
  
  public List<BasicDBObject> findBySubCodeAndStaffIdListWithNoDdbs(String subCode, List<String> staffIdList) {
    if (staffIdList == null || staffIdList.size() == 0)
      return null; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("staffId").in(staffIdList));
    query.addCriteria((CriteriaDefinition)(new Criteria()).orOperator(new Criteria[] { Criteria.where("ddbs").exists(false), 
            Criteria.where("ddbs").is(null), 
            Criteria.where("ddbs").is("") }));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findBySubCodeAndStaffIdList(String subCode, List<String> staffIdList) {
    if (staffIdList == null || staffIdList.size() == 0)
      return null; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("staffId").in(staffIdList));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findBySubCodeAndOrgIdList(String subCode, List<String> orgIdList) {
    if (orgIdList == null || orgIdList.size() == 0)
      return null; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("orgId").in(orgIdList));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findBySubCodeAndStaffIdListAndTenantId(String subCode, List<String> staffIdList, String tenantId) {
    if (staffIdList == null || staffIdList.size() == 0)
      return null; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").in(staffIdList));
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findBySubCodeAndStaffIdListAndTenantIdWithDensity(String subCode, List<String> staffIdList, String tenantId, HashMap<String, Object> densityMap) {
    if (staffIdList == null || staffIdList.size() == 0)
      return null; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").in(staffIdList));
    if (densityMap != null && densityMap.get("isStartDensity") != null && ((Boolean)densityMap.get("isStartDensity")).booleanValue())
      query.addCriteria(
          (CriteriaDefinition)Criteria.where(densityMap.get("codeField").toString())
          .nin((List)densityMap.get("levelList"))); 
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findBySubCodeAndStaffIdListAndTenantIdWithDensityCadre(String subCode, List<String> staffIdList, String tenantId, Query query) {
    if (staffIdList == null || staffIdList.size() == 0)
      return null; 
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").in(staffIdList));
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findBySubCodeWithNotDensity(String subCode, List<String> staffIdList, String tenantId, HashMap<String, Object> densityMap) {
    if (staffIdList == null || staffIdList.size() == 0)
      return null; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").in(staffIdList));
    if (densityMap != null && densityMap.get("isStartDensity") != null && ((Boolean)densityMap.get("isStartDensity")).booleanValue())
      query.addCriteria(
          (CriteriaDefinition)Criteria.where(densityMap.get("codeField").toString())
          .in((List)densityMap.get("levelList"))); 
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public Map<String, List<BasicDBObject>> findBySubCodeAndStaffIdListAndTenantId(String subCode, List<String> staffIdList, String tenantId, List<String> needFieldList) {
    Map<String, List<BasicDBObject>> staffExtMap = new HashMap<>();
    if (staffIdList == null || staffIdList.size() == 0)
      return staffExtMap; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").in(staffIdList));
    query.fields().include("tenantId");
    query.fields().include("staffId");
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    if (needFieldList != null && needFieldList.size() > 0)
      for (String s : needFieldList)
        query.fields().include(s);  
    DBCursor dbCursor = this.mongoTemplate.getCollection(subCode).find(query.getQueryObject(), query.getFieldsObject()).batchSize(1000);
    try {
      while (dbCursor.hasNext()) {
        List<BasicDBObject> list;
        BasicDBObject object = (BasicDBObject)dbCursor.next();
        if (staffExtMap.containsKey(object.get("staffId").toString())) {
          list = staffExtMap.get(object.get("staffId").toString());
        } else {
          list = new ArrayList<>();
        } 
        list.add(object);
        staffExtMap.put(object.get("staffId").toString(), list);
      } 
    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      if (dbCursor != null)
        dbCursor.close(); 
    } 
    return staffExtMap;
  }
  
  public Map<String, BasicDBObject> findNoSortBySubCodeAndStaffIdListAndTenantId(String subCode, List<String> staffIdList, String tenantId, List<String> needFieldList) {
    Map<String, BasicDBObject> staffExtMap = new HashMap<>();
    if (staffIdList == null || staffIdList.size() == 0)
      return staffExtMap; 
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").in(staffIdList));
    query.fields().include("tenantId");
    query.fields().include("staffId");
    if (needFieldList != null && needFieldList.size() > 0)
      for (String s : needFieldList)
        query.fields().include(s);  
    DBCursor dbCursor = this.mongoTemplate.getCollection(subCode).find(query.getQueryObject(), query.getFieldsObject()).batchSize(1000);
    try {
      while (dbCursor.hasNext()) {
        BasicDBObject object = (BasicDBObject)dbCursor.next();
        staffExtMap.put(object.get("staffId").toString(), object);
      } 
    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      if (dbCursor != null)
        dbCursor.close(); 
    } 
    return staffExtMap;
  }
  
  public Map<String, BasicDBObject> findNoSortBySubCodeAndStaffIdListAndTenantId(String subCode, List<String> staffIdList, String tenantId) {
    Map<String, BasicDBObject> staffExtMap = new HashMap<>();
    if (staffIdList == null || staffIdList.size() == 0)
      return staffExtMap; 
    BasicDBObject query = new BasicDBObject();
    query.put("tenantId", tenantId);
    query.append("staffId", new BasicDBObject("$in", staffIdList));
    DBCursor dbCursor = this.mongoTemplate.getCollection(subCode).find((DBObject)query);
    try {
      while (dbCursor.hasNext()) {
        BasicDBObject object = (BasicDBObject)dbCursor.next();
        staffExtMap.put(object.get("staffId").toString(), object);
      } 
    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      if (dbCursor != null)
        dbCursor.close(); 
    } 
    return staffExtMap;
  }
  
  public List<BasicDBObject> findByFieldAndList(String subCode, String fieldCode, List<String> list, HashMap<String, Object> densityMap) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where(fieldCode).in(list));
    if (densityMap != null && densityMap.get("isStartDensity") != null && ((Boolean)densityMap.get("isStartDensity")).booleanValue())
      query.addCriteria(
          (CriteriaDefinition)Criteria.where(densityMap.get("codeField").toString())
          .nin((List)densityMap.get("levelList"))); 
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findByFieldAndListAndTenantId(String subCode, String tenantId, String fieldCode, List<String> list, HashMap<String, Object> densityMap) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where(fieldCode).in(list));
    if (densityMap != null && densityMap.get("isStartDensity") != null && ((Boolean)densityMap.get("isStartDensity")).booleanValue())
      query.addCriteria(
          (CriteriaDefinition)Criteria.where(densityMap.get("codeField").toString())
          .nin((List)densityMap.get("levelList"))); 
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findByStaffIdAndSubCode(String staffId, String subCode) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("staffId").is(staffId));
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findByStaffIdAndSubCodeWithDensity(String staffId, String subCode, HashMap<String, Object> densityMap) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("staffId").is(staffId));
    if (densityMap != null && densityMap.get("isStartDensity") != null && ((Boolean)densityMap.get("isStartDensity")).booleanValue())
      query.addCriteria(
          (CriteriaDefinition)Criteria.where(densityMap.get("codeField").toString())
          .nin((List)densityMap.get("levelList"))); 
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public BasicDBObject getByStaffIdAndSubCode(String staffId, String subCode) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("staffId").is(staffId));
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
  }
  
  public Page<BasicDBObject> findPageByTenantId(String tenantId, String subCode, Integer pageIndex, Integer pageSize) {
    Query query = new Query((CriteriaDefinition)Criteria.where("tenantId").is(tenantId));
    return findPage(subCode, query, pageIndex, pageSize);
  }
  
  public Page<BasicDBObject> findPage(String document, Query condition, Pageable pageable) {
    long count = this.mongoTemplate.count(condition, document);
    if (condition.getSortObject() == null)
      condition.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" })); 
    List<BasicDBObject> list = null;
    if (count > (pageable.getPageNumber() * pageable.getPageSize())) {
      list = this.mongoTemplate.find(condition.with(pageable), BasicDBObject.class, document);
    } else {
      list = new ArrayList<>();
    } 
    return (Page<BasicDBObject>)new PageImpl(list, pageable, count);
  }
  
  public Page<BasicDBObject> findPage(String document, Query condition, Integer pageIndex, Integer pageSize) {
    long count = this.mongoTemplate.count(condition, document);
    VariablePageRequest variablePageRequest = PageHolder.buildRequest(pageIndex, pageSize);
    if (variablePageRequest.getSort() == null)
      condition.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" })); 
    List<BasicDBObject> list = null;
    if (count > (pageIndex.intValue() * pageSize.intValue())) {
      list = this.mongoTemplate.find(condition.with((Pageable)variablePageRequest), BasicDBObject.class, document);
    } else {
      list = new ArrayList<>();
    } 
    return (Page<BasicDBObject>)new PageImpl(list, (Pageable)variablePageRequest, count);
  }
  
  public Page<BasicDBObject> findStaffOneNewCreatTime(String document, Aggregation aggregation, Integer pageIndex, Integer pageSize, Aggregation countAggregation) {
    VariablePageRequest variablePageRequest = PageHolder.buildRequest(pageIndex, pageSize);
    List<BasicDBObject> list = null;
    AggregationResults<BasicDBObject> results = this.mongoTemplate.aggregate(aggregation, document, BasicDBObject.class);
    AggregationResults<BasicDBObject> countResults = this.mongoTemplate.aggregate(countAggregation, document, BasicDBObject.class);
    long total = 0L;
    if (!countResults.getMappedResults().isEmpty())
      total = ((BasicDBObject)countResults.getUniqueMappedResult()).getLong("total"); 
    list = results.getMappedResults();
    return (Page<BasicDBObject>)new PageImpl(list, (Pageable)variablePageRequest, total);
  }
  
  public List<BasicDBObject> findAllStaffOneNewCreatTime(String document, Aggregation aggregation) {
    List<BasicDBObject> list = null;
    AggregationResults<BasicDBObject> results = this.mongoTemplate.aggregate(aggregation, document, BasicDBObject.class);
    list = results.getMappedResults();
    return list;
  }
  
  public Page<BasicDBObject> findPage(String document, Query condition, Integer pageIndex, Integer pageSize, Sort sort) {
    long count = this.mongoTemplate.count(condition, document);
    VariablePageRequest variablePageRequest = PageHolder.buildRequest(pageIndex, pageSize);
    if (sort == null) {
      condition.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    } else {
      condition.with(sort);
    } 
    List<BasicDBObject> list = null;
    if (count > (pageIndex.intValue() * pageSize.intValue())) {
      list = this.mongoTemplate.find(condition.with((Pageable)variablePageRequest), BasicDBObject.class, document);
    } else {
      list = new ArrayList<>();
    } 
    return (Page<BasicDBObject>)new PageImpl(list, (Pageable)variablePageRequest, count);
  }
  
  public List<BasicDBObject> find(String document, Query condition) {
    return this.mongoTemplate.find(condition, BasicDBObject.class, document);
  }
  
  public List<BasicDBObject> findByTenantIdAndSubCode(String tenantId, String subCode, HashMap<String, Object> densityMap) {
    List<BasicDBObject> list = new ArrayList<>();
    BasicDBObject query = new BasicDBObject();
    AggregationOptions build = AggregationOptions.builder().allowDiskUse(Boolean.valueOf(true)).outputMode(AggregationOptions.OutputMode.CURSOR).build();
    List<DBObject> pipeline = new ArrayList<>();
    query.put("tenantId", tenantId);
    if (densityMap != null && densityMap.get("isStartDensity") != null && ((Boolean)densityMap.get("isStartDensity")).booleanValue())
      query.put(densityMap.get("codeField").toString(), new BasicDBObject("$nin", densityMap.get("levelList"))); 
    BasicDBObject basicDBObject1 = new BasicDBObject("showOrder", Integer.valueOf(1));
    BasicDBObject sort = new BasicDBObject("$sort", basicDBObject1);
    BasicDBObject basicDBObject2 = new BasicDBObject("$match", query);
    pipeline.add(sort);
    pipeline.add(basicDBObject2);
    Cursor cursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    while (cursor.hasNext())
      list.add((BasicDBObject)cursor.next()); 
    return list;
  }
  
  public List<BasicDBObject> findByTenantIdAndSubCode(String tenantId, String subCode, HashMap<String, Object> densityMap, List<String> orgIds) {
    List<BasicDBObject> list = new ArrayList<>();
    BasicDBObject query = new BasicDBObject();
    AggregationOptions build = AggregationOptions.builder().allowDiskUse(Boolean.valueOf(true)).outputMode(AggregationOptions.OutputMode.CURSOR).build();
    List<DBObject> pipeline = new ArrayList<>();
    query.put("tenantId", tenantId);
    if (densityMap != null && densityMap.get("isStartDensity") != null && ((Boolean)densityMap.get("isStartDensity")).booleanValue())
      query.put(densityMap.get("codeField").toString(), new BasicDBObject("$nin", densityMap.get("levelList"))); 
    if (orgIds != null && orgIds.size() > 0 && StringUtils.equals(subCode, "APR")) {
      orgIds.add((String)null);
      query.put("APR01", new BasicDBObject("$in", orgIds));
    } 
    BasicDBObject basicDBObject1 = new BasicDBObject("showOrder", Integer.valueOf(1));
    BasicDBObject sort = new BasicDBObject("$sort", basicDBObject1);
    BasicDBObject basicDBObject2 = new BasicDBObject("$match", query);
    pipeline.add(sort);
    pipeline.add(basicDBObject2);
    Cursor cursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    while (cursor.hasNext())
      list.add((BasicDBObject)cursor.next()); 
    return list;
  }
  
  public List<BasicDBObject> findByTenantIdAndSubCode(String tenantId, String subCode, HashMap<String, Object> densityMap, Date date) {
    List<BasicDBObject> list = new ArrayList<>();
    BasicDBObject query = new BasicDBObject();
    AggregationOptions build = AggregationOptions.builder().allowDiskUse(Boolean.valueOf(true)).outputMode(AggregationOptions.OutputMode.CURSOR).build();
    List<DBObject> pipeline = new ArrayList<>();
    query.put("tenantId", tenantId);
    query.put("createTs", new BasicDBObject("$gte", date));
    BasicDBObject orQuery = new BasicDBObject();
    orQuery.put("$or", new BasicDBObject[] { new BasicDBObject("createTs", new BasicDBObject("$gte", date)), new BasicDBObject("modifyTs", new BasicDBObject("$gte", date)) });
    query.put("$and", new BasicDBObject[] { new BasicDBObject("tenantId", tenantId), orQuery });
    if (densityMap != null && densityMap.get("isStartDensity") != null && ((Boolean)densityMap.get("isStartDensity")).booleanValue())
      query.put(densityMap.get("codeField").toString(), new BasicDBObject("$nin", densityMap.get("levelList"))); 
    BasicDBObject basicDBObject1 = new BasicDBObject("showOrder", Integer.valueOf(1));
    BasicDBObject sort = new BasicDBObject("$sort", basicDBObject1);
    BasicDBObject basicDBObject2 = new BasicDBObject("$match", query);
    pipeline.add(sort);
    pipeline.add(basicDBObject2);
    Cursor cursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    while (cursor.hasNext())
      list.add((BasicDBObject)cursor.next()); 
    return list;
  }
  
  public List<BasicDBObject> findFastByTenantIdAndSubCode(String tenantId, String subCode) {
    BasicDBObject query = new BasicDBObject();
    query.put("tenantId", tenantId);
    DBCursor dbCursor = this.mongoTemplate.getCollection(subCode).find((DBObject)query);
    List<BasicDBObject> list = new ArrayList<>();
    while (dbCursor.hasNext()) {
      BasicDBObject object = (BasicDBObject)dbCursor.next();
      list.add(object);
    } 
    return list;
  }
  
  public List<BasicDBObject> findFastByHcmIdAndSubCode(String subCode, String tenantId) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("hcmId").exists(true));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public BasicDBObject getLastMongoDataByTenantIdAndStaffId(String subCode, String tenantId, String staffId) {
    AggregationOptions build = AggregationOptions.builder().outputMode(AggregationOptions.OutputMode.CURSOR).allowDiskUse(Boolean.valueOf(true)).build();
    List<DBObject> pipeline = new ArrayList<>();
    BasicDBObject query = new BasicDBObject();
    query.put("tenantId", tenantId);
    query.put("staffId", staffId);
    BasicDBObject basicDBObject1 = new BasicDBObject("showOrder", Integer.valueOf(-1));
    BasicDBObject sort = new BasicDBObject("$sort", basicDBObject1);
    BasicDBObject basicDBObject2 = new BasicDBObject("$match", query);
    pipeline.add(sort);
    pipeline.add(basicDBObject2);
    BasicDBObject basicDBObject3 = new BasicDBObject("$skip", Integer.valueOf(0));
    BasicDBObject basicDBObject4 = new BasicDBObject("$limit", Integer.valueOf(1));
    pipeline.add(basicDBObject3);
    pipeline.add(basicDBObject4);
    Cursor dbCursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    List<BasicDBObject> list = new ArrayList<>();
    while (dbCursor.hasNext()) {
      BasicDBObject object = (BasicDBObject)dbCursor.next();
      list.add(object);
    } 
    return (list != null && list.size() > 0) ? list.get(0) : new BasicDBObject();
  }
  
  public List<BasicDBObject> findFastByTenantIdAndSubCodebySort(String tenantId, List<String> staffIds, String subCode) {
    BasicDBObject query = new BasicDBObject();
    AggregationOptions build = AggregationOptions.builder().outputMode(AggregationOptions.OutputMode.CURSOR).build();
    List<DBObject> pipeline = new ArrayList<>();
    query.put("tenantId", tenantId);
    query.append("staffId", new BasicDBObject("$in", staffIds));
    BasicDBObject basicDBObject1 = new BasicDBObject("showOrder", Integer.valueOf(1));
    BasicDBObject sort = new BasicDBObject("$sort", basicDBObject1);
    BasicDBObject basicDBObject2 = new BasicDBObject("$match", query);
    pipeline.add(sort);
    pipeline.add(basicDBObject2);
    Cursor dbCursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    List<BasicDBObject> list = new ArrayList<>();
    while (dbCursor.hasNext()) {
      BasicDBObject object = (BasicDBObject)dbCursor.next();
      list.add(object);
    } 
    return list;
  }
  
  public int findMaxShowOrderByTenantIdAndStaffIdAndSubCode(String tenantId, String staffId, String subCode) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").is(staffId));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    BasicDBObject basicDBObject = (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
    return (basicDBObject == null) ? 0 : ((basicDBObject.get("showOrder") == null) ? 0 : basicDBObject.getInt("showOrder"));
  }
  
  public int findMaxShowOrderByTenantIdAndSubCode(String tenantId, String subCode) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    BasicDBObject basicDBObject = (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
    return (basicDBObject == null) ? 0 : ((basicDBObject.get("showOrder") == null) ? 0 : basicDBObject.getInt("showOrder"));
  }
  
  public int findMaxShowOrderByTenantIdAndOrgIdAndSubCode(String tenantId, String orgId, String subCode) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("orgId").is(orgId));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    BasicDBObject basicDBObject = (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
    return (basicDBObject == null) ? 0 : ((basicDBObject.get("showOrder") == null) ? 0 : basicDBObject.getInt("showOrder"));
  }
  
  public int updateShowOrderByTenantIdAndStaffIdAndSubCode(String tenantId, String staffId, String subCode, String uuid) {
    Query query = new Query();
    Query queryByUuid = new Query();
    queryByUuid.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").is(staffId).and("_id").is(uuid));
    BasicDBObject byUuid = (BasicDBObject)this.mongoTemplate.findById(uuid, BasicDBObject.class, subCode);
    int showOrder = byUuid.getInt("showOrder");
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").is(staffId).and("showOrder").gte(Integer.valueOf(showOrder)));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    List<BasicDBObject> basicDBObjects = this.mongoTemplate.find(query, BasicDBObject.class, subCode);
    basicDBObjects.forEach(item -> item.put("showOrder", Integer.valueOf(item.getInt("showOrder") + 1)));
    saveOrUpdate(basicDBObjects, subCode);
    return showOrder;
  }
  
  public int updateShowOrderByTenantIdAndSubCode(String tenantId, String subCode, String uuid) {
    Query query = new Query();
    Query queryByUuid = new Query();
    queryByUuid.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("_id").is(uuid));
    BasicDBObject byUuid = (BasicDBObject)this.mongoTemplate.findById(uuid, BasicDBObject.class, subCode);
    int showOrder = byUuid.getInt("showOrder");
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("showOrder").gte(Integer.valueOf(showOrder)));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    List<BasicDBObject> basicDBObjects = this.mongoTemplate.find(query, BasicDBObject.class, subCode);
    basicDBObjects.forEach(item -> item.put("showOrder", Integer.valueOf(item.getInt("showOrder") + 1)));
    saveOrUpdate(basicDBObjects, subCode);
    return showOrder;
  }
  
  public int updateShowOrderByTenantIdAndOrgIdAndSubCode(String tenantId, String orgId, String subCode, String uuid) {
    Query query = new Query();
    Query queryByUuid = new Query();
    queryByUuid.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("orgId").is(orgId).and("_id").is(uuid));
    BasicDBObject byUuid = (BasicDBObject)this.mongoTemplate.findById(uuid, BasicDBObject.class, subCode);
    int showOrder = byUuid.getInt("showOrder");
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("orgId").is(orgId).and("showOrder").gte(Integer.valueOf(showOrder)));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    List<BasicDBObject> basicDBObjects = this.mongoTemplate.find(query, BasicDBObject.class, subCode);
    basicDBObjects.forEach(item -> item.put("showOrder", Integer.valueOf(item.getInt("showOrder") + 1)));
    saveOrUpdate(basicDBObjects, subCode);
    return showOrder;
  }
  
  public BasicDBObject findMaxShowOrderDataByTenantIdAndStaffIdAndSubCode(String tenantId, String staffId, String subCode) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").is(staffId));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    BasicDBObject basicDBObject = (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
    return basicDBObject;
  }
  
  public BasicDBObject findGtByStaffIdAndSubCodeAndShowOrder(String staffId, String subCode, int showOrder) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("staffId").is(staffId).and("showOrder").gt(Integer.valueOf(showOrder)));
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    query.limit(1);
    return (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
  }
  
  public BasicDBObject findGtByOrgIdAndSubCodeAndShowOrder(String orgId, String subCode, int showOrder) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("orgId").is(orgId).and("showOrder").gt(Integer.valueOf(showOrder)));
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    query.limit(1);
    return (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
  }
  
  public BasicDBObject findLtByStaffIdAndSubCodeAndShowOrder(String staffId, String subCode, int showOrder) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("staffId").is(staffId).and("showOrder").lt(Integer.valueOf(showOrder)));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    query.limit(1);
    return (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
  }
  
  public BasicDBObject findLtByOrgIdAndSubCodeAndShowOrder(String orgId, String subCode, int showOrder) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("orgId").is(orgId).and("showOrder").lt(Integer.valueOf(showOrder)));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    query.limit(1);
    return (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
  }
  
  public BasicDBObject findGtDataBySubCodeAndShowOrder(String fieldCode, String fieldValue, String subCode, int showOrder) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where(fieldCode).is(fieldValue).and("showOrder").gt(Integer.valueOf(showOrder)));
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    query.limit(1);
    return (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
  }
  
  public BasicDBObject findLtDataBySubCodeAndShowOrder(String fieldCode, String fieldValue, String subCode, int showOrder) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where(fieldCode).is(fieldValue).and("showOrder").lt(Integer.valueOf(showOrder)));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    query.limit(1);
    return (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
  }
  
  public BasicDBObject findDataBySubCodeAndThirdId(String subCode, String thirdId) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("thirdId").is(thirdId));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    query.limit(1);
    return (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
  }
  
  public BasicDBObject findByTenantIdAndThirdId(String subCode, String tenantId, String thirdId) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("thirdId").is(thirdId));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    query.limit(1);
    return (BasicDBObject)this.mongoTemplate.findOne(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findLastByStaffIdListAndSubCodeAndTenantId(List<String> staffIdList, String subCode, String tenantId, List<String> needFieldList) {
    List<BasicDBObject> list = new ArrayList<>();
    AggregationOptions build = AggregationOptions.builder().outputMode(AggregationOptions.OutputMode.CURSOR).build();
    List<DBObject> pipeline = new ArrayList<>();
    BasicDBObject basicDBObject1 = new BasicDBObject("_id", "$staffId");
    BasicDBObject basicDBObject2 = new BasicDBObject();
    BasicDBObject basicDBObject3 = new BasicDBObject();
    basicDBObject3.put("staffId", Integer.valueOf(1));
    basicDBObject3.put("showOrder", Integer.valueOf(-1));
    basicDBObject2.put("$sort", basicDBObject3);
    BasicDBObject basicDBObject4 = new BasicDBObject();
    if (needFieldList != null)
      for (String needField : needFieldList) {
        if (StringUtil.isEmpty(needField))
          continue; 
        BasicDBObject basicDBObject = new BasicDBObject();
        basicDBObject.put("$first", "$" + needField);
        basicDBObject1.put(needField, basicDBObject);
      }  
    basicDBObject4.put("$group", basicDBObject1);
    BasicDBObject basicDBObject5 = new BasicDBObject("$in", staffIdList);
    BasicDBObject basicDBObject6 = new BasicDBObject("staffId", basicDBObject5);
    basicDBObject6.put("tenantId", tenantId);
    BasicDBObject basicDBObject7 = new BasicDBObject("$match", basicDBObject6);
    pipeline.add(basicDBObject7);
    pipeline.add(basicDBObject2);
    pipeline.add(basicDBObject4);
    Cursor cursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    while (cursor.hasNext())
      list.add((BasicDBObject)cursor.next()); 
    return list;
  }
  
  public List<BasicDBObject> findLastBySubCodeAndTenantId(String subCode, String tenantId, List<String> needFieldList) {
    List<BasicDBObject> list = new ArrayList<>();
    AggregationOptions build = AggregationOptions.builder().outputMode(AggregationOptions.OutputMode.CURSOR).build();
    List<DBObject> pipeline = new ArrayList<>();
    BasicDBObject basicDBObject1 = new BasicDBObject("_id", "$staffId");
    BasicDBObject basicDBObject2 = new BasicDBObject();
    BasicDBObject basicDBObject3 = new BasicDBObject();
    basicDBObject3.put("staffId", Integer.valueOf(1));
    basicDBObject3.put("showOrder", Integer.valueOf(-1));
    basicDBObject2.put("$sort", basicDBObject3);
    BasicDBObject basicDBObject4 = new BasicDBObject();
    if (needFieldList != null)
      for (String needField : needFieldList) {
        if (StringUtil.isEmpty(needField))
          continue; 
        BasicDBObject basicDBObject = new BasicDBObject();
        basicDBObject.put("$first", "$" + needField);
        basicDBObject1.put(needField, basicDBObject);
      }  
    basicDBObject4.put("$group", basicDBObject1);
    BasicDBObject basicDBObject5 = new BasicDBObject("tenantId", tenantId);
    BasicDBObject basicDBObject6 = new BasicDBObject("$match", basicDBObject5);
    pipeline.add(basicDBObject6);
    pipeline.add(basicDBObject2);
    pipeline.add(basicDBObject4);
    Cursor cursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    while (cursor.hasNext())
      list.add((BasicDBObject)cursor.next()); 
    return list;
  }
  
  public List<String> findListBySubCodeAndFieldCodeAndFieldValue(String tenantId, String subCode, String fieldCode, String fieldValue) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and(fieldCode).is(fieldValue));
    return this.mongoTemplate.getCollection(subCode).distinct("staffId", query.getQueryObject());
  }
  
  public List<BasicDBObject> findBySubCodeAndFieldCodeAndFieldValue(String subCode, String fieldCode, String fieldValue) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where(fieldCode).is(fieldValue));
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findSecondSonListLastByStaffIdListAndSubCodeAndTenantId(List<String> staffIdList, String subCode, String tenantId, List<String> needFieldList, String infoCodeValue) {
    List<BasicDBObject> list = new ArrayList<>();
    AggregationOptions build = AggregationOptions.builder().outputMode(AggregationOptions.OutputMode.CURSOR).build();
    List<DBObject> pipeline = new ArrayList<>();
    BasicDBObject basicDBObject1 = new BasicDBObject("_id", "$staffId");
    BasicDBObject basicDBObject2 = new BasicDBObject();
    BasicDBObject basicDBObject3 = new BasicDBObject();
    basicDBObject3.put("staffId", Integer.valueOf(1));
    basicDBObject3.put("showOrder", Integer.valueOf(-1));
    basicDBObject2.put("$sort", basicDBObject3);
    BasicDBObject basicDBObject4 = new BasicDBObject();
    if (needFieldList != null)
      for (String needField : needFieldList) {
        if (StringUtil.isEmpty(needField))
          continue; 
        BasicDBObject basicDBObject = new BasicDBObject();
        basicDBObject.put("$first", "$" + needField);
        basicDBObject1.put(needField, basicDBObject);
      }  
    basicDBObject4.put("$group", basicDBObject1);
    BasicDBObject basicDBObject5 = new BasicDBObject("$in", staffIdList);
    BasicDBObject basicDBObject6 = new BasicDBObject("staffId", basicDBObject5);
    basicDBObject6.put("tenantId", tenantId);
    basicDBObject6.put(needFieldList.get(0), infoCodeValue);
    BasicDBObject basicDBObject7 = new BasicDBObject("$match", basicDBObject6);
    pipeline.add(basicDBObject7);
    pipeline.add(basicDBObject2);
    pipeline.add(basicDBObject4);
    Cursor cursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    while (cursor.hasNext())
      list.add((BasicDBObject)cursor.next()); 
    return list;
  }
  
  public List<BasicDBObject> findNewLastByStaffIdListAndSubCodeAndTenantId(List<String> staffIdList, String subCode, String tenantId, List<String> needFieldList) {
    List<BasicDBObject> list = new ArrayList<>();
    AggregationOptions build = AggregationOptions.builder().outputMode(AggregationOptions.OutputMode.CURSOR).build();
    List<DBObject> pipeline = new ArrayList<>();
    BasicDBObject basicDBObject1 = new BasicDBObject("_id", "$staffId");
    BasicDBObject basicDBObject2 = new BasicDBObject();
    BasicDBObject basicDBObject3 = new BasicDBObject();
    basicDBObject3.put("staffId", Integer.valueOf(1));
    basicDBObject3.put("showOrder", Integer.valueOf(-1));
    basicDBObject2.put("$sort", basicDBObject3);
    BasicDBObject basicDBObject4 = new BasicDBObject();
    if (needFieldList != null)
      for (String needField : needFieldList) {
        if (StringUtil.isEmpty(needField))
          continue; 
        BasicDBObject basicDBObject = new BasicDBObject();
        basicDBObject.put("$first", "$" + needField);
        basicDBObject1.put(needField, basicDBObject);
      }  
    basicDBObject4.put("$group", basicDBObject1);
    BasicDBObject basicDBObject5 = new BasicDBObject("$in", staffIdList);
    BasicDBObject basicDBObject6 = new BasicDBObject("staffId", basicDBObject5);
    basicDBObject6.put("tenantId", tenantId);
    BasicDBObject basicDBObject7 = new BasicDBObject("$match", basicDBObject6);
    pipeline.add(basicDBObject7);
    pipeline.add(basicDBObject2);
    pipeline.add(basicDBObject4);
    Cursor cursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    while (cursor.hasNext())
      list.add((BasicDBObject)cursor.next()); 
    return list;
  }
  
  public List<BasicDBObject> findByTenantIdAndStaffIdAndSubCode(String tenantId, String staffId, String subCode) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").is(staffId));
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findBySubCodeAndPageInfo(String subCode, String fieldCode, String fieldValue, String orderFieldCode, Sort.Direction direction, Integer skip, Integer pageSize, String sSearch) {
    List<BasicDBObject> result = new ArrayList<>();
    if (StringUtil.isEmpty(subCode))
      return result; 
    Query query = new Query();
    if (StringUtil.isEmpty(sSearch)) {
      query.addCriteria((CriteriaDefinition)Criteria.where(fieldCode).is(fieldValue));
    } else {
      Pattern pattern = Pattern.compile("^.*" + sSearch + ".*$", 2);
      query.addCriteria((CriteriaDefinition)Criteria.where(fieldCode).is(fieldValue).and("A0101").regex(pattern));
    } 
    query.with(new Sort(Sort.Direction.ASC, new String[] { orderFieldCode }));
    query.skip(skip.intValue());
    query.limit(pageSize.intValue());
    result = this.mongoTemplate.find(query, BasicDBObject.class, subCode);
    return result;
  }
  
  public long countBySubCodeAndFieldCodeAndFieldValue(String subCode, String fieldCode, String fieldValue, String sSearch) {
    Query query = new Query();
    if (StringUtil.isEmpty(sSearch)) {
      query.addCriteria((CriteriaDefinition)Criteria.where(fieldCode).is(fieldValue));
    } else {
      Pattern pattern = Pattern.compile("^.*" + sSearch + ".*$", 2);
      query.addCriteria((CriteriaDefinition)Criteria.where(fieldCode).is(fieldValue).and("A0101").regex(pattern));
    } 
    return this.mongoTemplate.count(query, subCode);
  }
  
  public List<String> findListBySubCodeAndFieldCodeAndFieldValueList(String tenantId, String subCode, String fieldCode, List<String> fieldValueList) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and(fieldCode).in(fieldValueList));
    if ("A0132".equals(fieldCode)) {
      Query querys = new Query();
      Criteria criteria = new Criteria();
      String value = "";
      for (int i = 0; i < fieldValueList.size(); i++) {
        if (i == fieldValueList.size() - 1) {
          value = value + (String)fieldValueList.get(i);
        } else {
          value = value + (String)fieldValueList.get(i) + "|";
        } 
      } 
      String valueTemp = "";
      if (value.lastIndexOf("0") == value.length() - 1 && "A0405".equals(fieldCode)) {
        valueTemp = value.substring(0, 1);
      } else {
        valueTemp = value;
      } 
      List<Criteria> criteriasTemp = new ArrayList<>();
      String[] values = valueTemp.split("\\|");
      for (String s : values) {
        Criteria c = new Criteria();
        Pattern leftPattern = Pattern.compile("^" + s + ".*|.*\\|" + s + ".*");
        criteriasTemp.add(c.and(fieldCode).regex(leftPattern));
      } 
      criteria.exists(true).orOperator(criteriasTemp.<Criteria>toArray(new Criteria[criteriasTemp.size()]));
      criteria.and("tenantId").is(tenantId);
      List<BasicDBObject> basicDBObjects = this.mongoTemplate.find(querys.addCriteria((CriteriaDefinition)criteria), BasicDBObject.class, subCode);
      List<String> staffIds = (List<String>)basicDBObjects.stream().map(obj -> obj.getString("staffId")).collect(Collectors.toList());
      return staffIds;
    } 
    return this.mongoTemplate.getCollection(subCode).distinct("staffId", query.getQueryObject());
  }
  
  public void saveListByFieldCodeListAndFieldValue(String tenantId, HashMap<String, List<String>> subKeyMap, String fieldValue, String newFieldValue) {
    for (String key : subKeyMap.keySet()) {
      for (int i = 0; i < ((List)subKeyMap.get(key)).size(); i++) {
        Query query = new Query();
        Criteria criteria = new Criteria();
        Criteria.where("tenantId").is(tenantId);
        query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId));
        if (!StringUtils.isEmpty(newFieldValue) || !((String)((List<String>)subKeyMap.get(key)).get(i)).contains("Text")) {
          query.addCriteria((CriteriaDefinition)Criteria.where(((List<String>)subKeyMap.get(key)).get(i)).is(fieldValue));
          List<BasicDBObject> ObjectList = this.mongoTemplate.find(query, BasicDBObject.class, key);
          List<BasicDBObject> updateList = new ArrayList<>();
          for (BasicDBObject basicDBObject : ObjectList) {
            basicDBObject.put(((List)subKeyMap.get(key)).get(i), newFieldValue);
            updateList.add(basicDBObject);
            if (updateList.size() == 1000) {
              saveOrUpdate(updateList, key);
              updateList.clear();
            } 
          } 
          if (updateList.size() > 0)
            saveOrUpdate(ObjectList, key); 
        } 
      } 
    } 
  }
  
  public void saveListByFieldCodeListAndFieldValue(String tenantId, HashMap<String, List<String>> subKeyMap, String fieldValue, String newFieldValue, String uuid) {
    for (String key : subKeyMap.keySet()) {
      for (int i = 0; i < ((List)subKeyMap.get(key)).size(); i++) {
        Query query = new Query();
        Criteria criteria = new Criteria();
        Criteria.where("tenantId").is(tenantId);
        query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId));
        if (!StringUtils.isEmpty(newFieldValue) || !((String)((List<String>)subKeyMap.get(key)).get(i)).contains("Text")) {
          query.addCriteria((CriteriaDefinition)Criteria.where(((List<String>)subKeyMap.get(key)).get(i)).is(uuid));
          List<BasicDBObject> ObjectList = this.mongoTemplate.find(query, BasicDBObject.class, key);
          List<BasicDBObject> updateList = new ArrayList<>();
          for (BasicDBObject basicDBObject : ObjectList) {
            if (StringUtils.isBlank(newFieldValue)) {
              basicDBObject.put((String)((List<String>)subKeyMap.get(key)).get(i) + "Text", "");
              basicDBObject.put(((List)subKeyMap.get(key)).get(i), "");
              basicDBObject.put((String)((List<String>)subKeyMap.get(key)).get(i) + "Code", "");
            } else {
              basicDBObject.put((String)((List<String>)subKeyMap.get(key)).get(i) + "Text", newFieldValue);
            } 
            updateList.add(basicDBObject);
            if (updateList.size() == 1000) {
              saveOrUpdate(updateList, key);
              updateList.clear();
            } 
          } 
          if (updateList.size() > 0)
            saveOrUpdate(ObjectList, key); 
        } 
      } 
    } 
  }
  
  public List<BasicDBObject> findByStaffIds(String subCode, Set<String> fields, Set<String> staffIds) {
    if (staffIds.isEmpty())
      return new ArrayList<>(); 
    BasicDBObject basicDBObject = new BasicDBObject();
    if (fields != null && !fields.isEmpty())
      for (String f : fields)
        basicDBObject.put(f, Boolean.valueOf(true));  
    BasicQuery query = new BasicQuery((DBObject)new BasicDBObject(), (DBObject)basicDBObject);
    query.addCriteria((CriteriaDefinition)Criteria.where("staffId").in(staffIds));
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find((Query)query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findByStaffIds(String subCode, Set<String> fields, String tenantId, Set<String> staffIds) {
    if (staffIds.isEmpty())
      return new ArrayList<>(); 
    BasicDBObject basicDBObject = new BasicDBObject();
    if (fields != null && !fields.isEmpty())
      for (String f : fields)
        basicDBObject.put(f, Boolean.valueOf(true));  
    BasicQuery query = new BasicQuery((DBObject)new BasicDBObject(), (DBObject)basicDBObject);
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").in(staffIds));
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find((Query)query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findBySubCodeAndFields(String subCode, Set<String> fields, String tenantId) {
    BasicDBObject basicDBObject = new BasicDBObject();
    if (fields != null && !fields.isEmpty())
      for (String f : fields)
        basicDBObject.put(f, Boolean.valueOf(true));  
    BasicQuery query = new BasicQuery((DBObject)new BasicDBObject(), (DBObject)basicDBObject);
    return this.mongoTemplate.find((Query)query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findByStaffIds(String subCode, Set<String> fields, String tenantId, Set<String> staffIds, Sort.Direction orderDirect) {
    if (staffIds.isEmpty())
      return new ArrayList<>(); 
    BasicDBObject basicDBObject = new BasicDBObject();
    BasicQuery query = new BasicQuery((DBObject)new BasicDBObject(), (DBObject)basicDBObject);
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").in(staffIds));
    query.with(new Sort(orderDirect, new String[] { "showOrder" }));
    return this.mongoTemplate.find((Query)query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findByStaffIdsNoDdbs(String subCode, Set<String> fields, String tenantId, Set<String> staffIds, Sort.Direction orderDirect) {
    if (staffIds.isEmpty())
      return new ArrayList<>(); 
    BasicDBObject basicDBObject = new BasicDBObject();
    BasicQuery query = new BasicQuery((DBObject)new BasicDBObject(), (DBObject)basicDBObject);
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").in(staffIds));
    query.with(new Sort(orderDirect, new String[] { "showOrder" }));
    query.addCriteria((CriteriaDefinition)(new Criteria()).orOperator(new Criteria[] { Criteria.where("ddbs").exists(false), 
            Criteria.where("ddbs").is(null), 
            Criteria.where("ddbs").is("") }));
    return this.mongoTemplate.find((Query)query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findStaffSubByStaffIdsForInven(String subCode, Set<String> fields, String tenantId, Set<String> staffIds, HashMap<String, Object> densityMap, List<String> orgIds) {
    if (staffIds.isEmpty())
      return new ArrayList<>(); 
    BasicDBObject basicDBObject1 = new BasicDBObject();
    BasicDBObject condition = new BasicDBObject();
    condition.put("tenantId", tenantId);
    condition.put("staffId", new BasicDBObject("$in", staffIds));
    if (densityMap != null && densityMap.get("isStartDensity") != null && ((Boolean)densityMap.get("isStartDensity")).booleanValue())
      condition.put(densityMap.get("codeField").toString(), new BasicDBObject("$nin", densityMap.get("levelList"))); 
    if (orgIds != null && orgIds.size() > 0 && StringUtils.equals(subCode, "APR")) {
      orgIds.add((String)null);
      condition.put("APR01", new BasicDBObject("$in", orgIds));
    } 
    BasicQuery query = new BasicQuery((DBObject)condition, (DBObject)basicDBObject1);
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find((Query)query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findOrgSubByOrgIdsForInven(String subCode, Set<String> fields, String tenantId, Set<String> orgIds, HashMap<String, Object> densityMap) {
    if (orgIds.isEmpty())
      return new ArrayList<>(); 
    BasicDBObject basicDBObject1 = new BasicDBObject();
    if (fields != null && !fields.isEmpty())
      for (String f : fields)
        basicDBObject1.put(f, Boolean.valueOf(true));  
    BasicDBObject condition = new BasicDBObject();
    condition.put("tenantId", tenantId);
    condition.put("orgId", new BasicDBObject("$in", orgIds));
    if (densityMap != null && densityMap.get("isStartDensity") != null && ((Boolean)densityMap.get("isStartDensity")).booleanValue())
      condition.put(densityMap.get("codeField").toString(), new BasicDBObject("$nin", densityMap.get("levelList"))); 
    BasicQuery query = new BasicQuery((DBObject)condition, (DBObject)basicDBObject1);
    query.with(new Sort(Sort.Direction.DESC, new String[] { "showOrder" }));
    return this.mongoTemplate.find((Query)query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findLastByStaffIds(String subCode, Set<String> fields, String tenantId, Set<String> staffIds, HashMap<String, Object> densityMap, List<String> orgIds) {
    if (staffIds.isEmpty())
      return new ArrayList<>(); 
    AggregationOptions build = AggregationOptions.builder().outputMode(AggregationOptions.OutputMode.CURSOR).build();
    List<DBObject> pipeline = new ArrayList<>();
    BasicDBObject basicDBObject1 = new BasicDBObject("staffId", new BasicDBObject("$in", staffIds));
    basicDBObject1.put("tenantId", tenantId);
    if (densityMap == null || densityMap.get("isStartDensity") == null || ((Boolean)densityMap.get("isStartDensity")).booleanValue());
    if (orgIds != null && orgIds.size() > 0 && StringUtils.equals(subCode, "APR")) {
      orgIds.add((String)null);
      basicDBObject1.put("APR01", new BasicDBObject("$in", orgIds));
    } 
    pipeline.add(new BasicDBObject("$match", basicDBObject1));
    BasicDBObject basicDBObject2 = new BasicDBObject();
    basicDBObject2.put("staffId", Integer.valueOf(1));
    basicDBObject2.put("showOrder", Integer.valueOf(-1));
    pipeline.add(new BasicDBObject("$sort", basicDBObject2));
    BasicDBObject basicDBObject3 = new BasicDBObject("_id", "$staffId");
    if (fields != null)
      for (String nf : fields) {
        if (StringUtils.isBlank(nf))
          continue; 
        basicDBObject3.put(nf, new BasicDBObject("$first", "$" + nf));
      }  
    if (!basicDBObject3.containsField("staffId"))
      basicDBObject3.put("staffId", new BasicDBObject("$first", "$staffId")); 
    pipeline.add(new BasicDBObject("$group", basicDBObject3));
    Cursor cursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    List<BasicDBObject> list = new ArrayList<>();
    while (cursor.hasNext())
      list.add((BasicDBObject)cursor.next()); 
    return list;
  }
  
  public List<BasicDBObject> findListBySubCodeAndPartTimeJobIdAndStaffId(String tenantId, String subCode, String partTimeJobId, String staffId) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId)
        .and("partTimeJobId").is(partTimeJobId)
        .and("staffId").is(staffId));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findListBySubCodeAndOrgId(String tenantId, String subCode, String orgId) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("orgId").is(orgId));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<BasicDBObject> findListBySubCodeAndStaffId(String tenantId, String subCode, String orgId) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").is(orgId));
    return this.mongoTemplate.find(query, BasicDBObject.class, subCode);
  }
  
  public List<String> findHasDataSubCodesByStaffId(String tenantId, String staffId, Set<String> subCodes) {
    if (subCodes.isEmpty())
      return new ArrayList<>(); 
    BasicDBObject basicDBObject = new BasicDBObject();
    basicDBObject.put("_id", Boolean.valueOf(true));
    BasicQuery query = new BasicQuery((DBObject)new BasicDBObject(), (DBObject)basicDBObject);
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("staffId").is(staffId));
    List<String> hasDataCodes = new ArrayList<>();
    for (String subCode : subCodes) {
      String id = (String)this.mongoTemplate.findOne((Query)query, String.class, subCode);
      if (StringUtils.isBlank(id))
        continue; 
      hasDataCodes.add(subCode);
    } 
    return hasDataCodes;
  }
  
  public List<BasicDBObject> findPosByTypes(Set<String> fields, String tenantId, Set<String> typeCodeItemIds) {
    if (typeCodeItemIds.isEmpty())
      return new ArrayList<>(); 
    BasicQuery query = null;
    if (fields != null && !fields.isEmpty()) {
      BasicDBObject basicDBObject = new BasicDBObject();
      for (String f : fields)
        basicDBObject.put(f, Boolean.valueOf(true)); 
      query = new BasicQuery((DBObject)new BasicDBObject(), (DBObject)basicDBObject);
    } else {
      query = new BasicQuery((DBObject)new BasicDBObject());
    } 
    query.addCriteria((CriteriaDefinition)Criteria.where("tenantId").is(tenantId).and("orgDeleteFlag").is("0").and("K0114").in(typeCodeItemIds));
    query.with(new Sort(Sort.Direction.ASC, new String[] { "showOrder" }));
    return this.mongoTemplate.find((Query)query, BasicDBObject.class, "K01");
  }
  
  public Map<String, Integer> countExistsByFieldCodeAndTids(List<String> tenantIdList, String subCode, String fieldCode) {
    Map<String, Integer> result = new HashMap<>();
    BasicDBObject query = new BasicDBObject();
    AggregationOptions build = AggregationOptions.builder().outputMode(AggregationOptions.OutputMode.CURSOR).build();
    List<String> nullList = new ArrayList();
    nullList.add("");
    nullList.add("null");
    nullList.add((String)null);
    List<DBObject> pipeline = new ArrayList<>();
    query.append("tenantId", new BasicDBObject("$in", tenantIdList));
    query.append("staffBase", new BasicDBObject("$ne", "2"));
    query.append(fieldCode, (new BasicDBObject("$exists", Boolean.valueOf(true))).append("$nin", nullList));
    BasicDBObject basicDBObject1 = new BasicDBObject("$match", query);
    pipeline.add(basicDBObject1);
    BasicDBObject group = new BasicDBObject();
    group.append("_id", "$tenantId");
    group.append("count", new BasicDBObject("$sum", Integer.valueOf(1)));
    BasicDBObject basicDBObject2 = new BasicDBObject("$group", group);
    pipeline.add(basicDBObject2);
    Cursor dbCursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    while (dbCursor.hasNext()) {
      BasicDBObject object = (BasicDBObject)dbCursor.next();
      result.put(object.getString("_id"), Integer.valueOf(object.getInt("count", 0)));
    } 
    return result;
  }
  
  public void changeFieldCode(String subCode, String oldFieldCode, String newFieldCode) {
    Query query = new Query();
    Update update = new Update();
    update.rename(oldFieldCode, newFieldCode);
    this.mongoTemplate.updateMulti(query, update, subCode);
  }
  
  public void clearFieldData(String subCode, String fieldCode) {
    Query query = new Query();
    Update update = new Update();
    update.unset(fieldCode);
    this.mongoTemplate.updateMulti(query, update, subCode);
  }
  
  public void deleteByProjectIdAndStaffId(String projectId, String objectId, String subCode) {
    Query query = new Query();
    query.addCriteria((CriteriaDefinition)Criteria.where("projectId").is(projectId).and("staffId").is(objectId));
    this.mongoTemplate.remove(query, subCode);
  }
  
  public List<BasicDBObject> findLastRealIdByStaffIdListAndSubCodeAndTenantId(List<String> staffIdList, String subCode, String tenantId, List<String> needFieldList) {
    List<BasicDBObject> list = new ArrayList<>();
    AggregationOptions build = AggregationOptions.builder().outputMode(AggregationOptions.OutputMode.CURSOR).build();
    List<DBObject> pipeline = new ArrayList<>();
    BasicDBObject basicDBObject1 = new BasicDBObject("_id", "$staffId");
    basicDBObject1.put("realId", new BasicDBObject("$first", "$_id"));
    BasicDBObject basicDBObject2 = new BasicDBObject();
    BasicDBObject basicDBObject3 = new BasicDBObject();
    basicDBObject3.put("staffId", Integer.valueOf(1));
    basicDBObject3.put("showOrder", Integer.valueOf(-1));
    basicDBObject2.put("$sort", basicDBObject3);
    BasicDBObject basicDBObject4 = new BasicDBObject();
    if (needFieldList != null)
      for (String needField : needFieldList) {
        if (StringUtil.isEmpty(needField))
          continue; 
        BasicDBObject basicDBObject = new BasicDBObject();
        basicDBObject.put("$first", "$" + needField);
        basicDBObject1.put(needField, basicDBObject);
      }  
    basicDBObject4.put("$group", basicDBObject1);
    BasicDBObject basicDBObject5 = new BasicDBObject("$in", staffIdList);
    BasicDBObject basicDBObject6 = new BasicDBObject("staffId", basicDBObject5);
    basicDBObject6.put("tenantId", tenantId);
    BasicDBObject basicDBObject7 = new BasicDBObject("$match", basicDBObject6);
    pipeline.add(basicDBObject7);
    pipeline.add(basicDBObject2);
    pipeline.add(basicDBObject4);
    Cursor cursor = this.mongoTemplate.getCollection(subCode).aggregate(pipeline, build);
    while (cursor.hasNext()) {
      BasicDBObject object = (BasicDBObject)cursor.next();
      object.put("_id", object.get("realId"));
      list.add(object);
    } 
    return list;
  }
}
